org &8000
nolist

txt_output equ &bb5a
km_wait_char equ &bb06
txt_set_cursor equ &bb75
scr_set_mode equ &bc0e
km_read_char equ &bb09
km_read_key equ &bb1b
mc_wait_flyback equ &bd19

;;==================================================================

start:
call cls
ld hl,menu_msg
call print_msg

call exe_menu
jp start

menu_msg:
defb "Disc drive and disc controller tests",10,13,10,13

defb "0. Quit",10,13
defb "1. Drive motor on/off",10,13
defb "2. Drive write protect on/off",10,13
defb "3. Drive Ready status test",10,13
defb "4. Drive seek test",10,13
defb 0



;;==================================================================

exe_menu:
call debounce_char
cp "0"
jp z,0			;; reset
cp "1"
jp z,drive_motor_test
cp "2"
jp z,drive_write_protect_test
cp "3"
jp z,drive_ready_test
cp "4"
jp z,drive_seek_test
ret


;;==================================================================
drive_seek_test:
call cls
ld hl,drive_seek_test_msg1
call print_msg
call pick_drive

call cls
ld hl,drive_seek_test_msg1
call print_msg
;; wait for it to be ready
call wait_drive_ready
ret z

call fdc_recalibrate
jr nc,drive_seek_test2

ld hl,drive_seek_fail_msg
call print_msg
call debounce_char
ret

drive_seek_test2:
xor a
ld (track),a

call cls

ld hl,drive_seek_test_msg1
call print_msg

call show_drive

ld hl,drive_seek_test_msg2
call print_msg

drive_seek_test_loop:
ld h,1
ld l,13
call txt_set_cursor
call show_track

call debounce_char
cp 'Q'
jr z,dst_ti
cp 'q'
jr z,dst_ti
cp 'A'
jr z,dst_td
cp 'a'
jr z,dst_td
cp &20
ret z
jp drive_seek_test_loop

dst_ti:
ld a,(track)
inc a
jr drive_seek_do

dst_td:
ld a,(track)
or a
jr z,drive_seek_test_loop
dec a
jr drive_seek_do

drive_seek_do:
ld (track),a
call fdc_seek
jr drive_seek_test_loop

drive_seek_fail_msg:
defb "Failed to seek to track 0. Press any key to return to menu",0

drive_seek_test_msg1:
defb "Drive seek test",10,13,10,13,0

drive_seek_test_msg2:
defb 10,13,10,13,"Keep test disc in the drive",10,13,10,13

defb "Use Q to increment (+) track number",10,13
defb "Use A to decrement (-) track number",10,13
defb "Use ESC to quit test",10,13
defb 10,13
defb "Listen to ensure drive performs step",10,13
defb 0

show_track:
ld hl,track_msg
call print_msg
ld a,(track)
ld l,a
ld h,0
call print_dec2
ret

track_msg:
defb "Track is ",0


print_dec:
LD    DE,10000
call print_digit
LD    DE,1000
call print_digit
LD    DE,100
call print_digit
print_dec2:
LD    DE,10
call print_digit
LD    DE,1
print_digit:
LD    B,"0"
print_digit2:
AND   A
SBC   HL,DE
JP    M,print_digit3       ;; -ve value
INC   B
JR    print_digit2
print_digit3: ADD   HL,DE        ;; restore to +ve
LD    A,B
jp txt_output

;;==================================================================
wait_drive_ready:
;; show the drive
call show_drive
call crlf
call crlf
;; ask user to insert disc into drive
ld hl,insert_disc_txt
call print_msg

call debounce_key

;; turn on motor
ld a,1
call set_motor

ld hl,spin_up_txt
call print_msg

;; wait for drive to become ready
ld b,50*3
call wait_vsyncs

call sense_drive_status
ld hl,result_data
bit 5,(hl)
ret nz

call show_ready

ld hl,ready_failed_txt
call print_msg

call debounce_key

xor a
ret

ready_failed_txt:
defb "Drive failed to be ready. Press any key to quit test",0

insert_disc_txt:
defb "Insert disc into drive and press any key",0

spin_up_txt:
defb 10,13,10,13,"Disc motor on, waiting for it to spin up and speed to stabalise",0

;;==================================================================
drive_write_protect_test:
call cls
ld hl,drive_wp_test_msg1
call print_msg
call pick_drive

call cls
ld hl,drive_wp_test_msg1
call print_msg

call show_drive

ld hl,drive_wp_test_msg2
call print_msg

drive_write_protect_test_loop:
ld b,25
call wait_vsyncs

;; report write protect to screen
ld h,1
ld l,17
call txt_set_cursor
call show_write_protect

call km_read_key
jr nc,drive_write_protect_test_loop
cp &20
ret z
jr drive_write_protect_test_loop

drive_wp_test_msg1:
defb "Disc drive write protect test",10,13,10,13
defb 0

drive_wp_test_msg2:
defb 10,13,10,13,"Checking 'write protect' ON:",10,13,10,13
defb "1. Set the write protect tab on a disc to ON",10,13
defb "2. Insert the disc into the test drive",10,13
defb "3. Drive should report write protect ON",10,13
defb 10,13
defb "Checking 'write protect' OFF:",10,13,10,13
defb "1. Set the write protect tab on a disc to OFF",10,13
defb "2. Insert the disc into the test drive",10,13
defb "3. Drive should report write protect OFF"
defb 0

;;==================================================================

show_write_protect:
ld hl,write_protect_txt
call print_msg

call sense_drive_status

ld hl,result_data
bit 6,(hl)
ld hl,off_txt
jr z,swp2
ld hl,on_txt
swp2:
jp print_msg

write_protect_txt:
defb "Write protect is ",0

;;==================================================================
drive_ready_test:
call cls
ld hl,drive_ready_test_msg1
call print_msg
call pick_drive

ld a,1
call set_motor

call cls
ld hl,drive_ready_test_msg1
call print_msg

call show_drive

ld hl,drive_ready_test_msg2
call print_msg

drive_ready_test_loop:
ld b,25
call wait_vsyncs

;; report ready protect to screen
ld h,1
ld l,17
call txt_set_cursor

call show_ready

call km_read_key
jr nc,drive_ready_test_loop
cp &20
ret z
jr drive_ready_test_loop


show_ready:
ld hl,drive_is_msg
call print_msg

call sense_drive_status
ld hl,result_data
bit 5,(hl)
ld hl,not_ready_txt
jr z,show_ready2
ld hl,ready_txt
show_ready2:
jp print_msg

ready_txt:
defb "Ready    ",0
not_ready_txt:
defb "Not Ready",0

;;==================================================================


;;-----------------------------------------------

drive_ready_test_msg1:
defb "Disc drive ready test",10,13,10,13
defb 0

drive_ready_test_msg2:
defb 10,13,10,13,"Checking 'ready':",10,13,10,13
defb "1. Insert a disc into the test drive. ",10,13
defb "2. Wait a few seconds",10,13
defb "3. Drive should report ready",10,13
defb 10,13
defb "Checking 'not ready':",10,13,10,13
defb "1. Remove disc from drive.",10,13
defb "2. Wait a few seconds",10,13
defb "3. Drive should report not ready",10,13
defb 0

;;==================================================================
drive_motor_test:
call reset_motor
call cls

ld hl,drive_motor_test_msg
call print_msg

dm_rfsh_ui:
ld h,1
ld l,8
call txt_set_cursor
ld hl,motor_msg
call print_msg
ld a,(motor)
or a
ld hl,off_txt
jr z,dm_rfsh_ui2
ld hl,on_txt
dm_rfsh_ui2:
call print_msg

drive_motor_test_loop:
call debounce_char
cp &20
jr z,toggle_motor

;; if esc pressed, reset motor and quit test
cp &1b
jr z,reset_motor

jr drive_motor_test_loop

;;-----------------------------------------------

toggle_motor:
ld a,(motor)
xor 1
call set_motor
jr dm_rfsh_ui

;;-----------------------------------------------

reset_motor:
xor a

set_motor:
ld (motor),a

;;-----------------------------------------------

refresh_motor:
ld a,(motor)
ld bc,&fa7e
out (c),a
ret


motor:
defb 0

motor_msg:
defb "Drive motor is ",0


;;-----------------------------------------------

drive_motor_test_msg:
defb "Disc drive motor test",10,13,10,13

defb "Turn the motor on/off and listen.",10,13
defb "Press SPACE to turn drive motor on/off.",10,13
defb "Press ESC to quit this test",10,13,10,13
defb 0

;;==================================================================
;; Pick a drive
pick_drive:
ld hl,drive_msg
call print_msg

call debounce_char
cp "A"
ld c,0
jr z,set_drive
cp "a"
ld c,0
jr z,set_drive
cp "B"
ld c,1
jr z,set_drive
cp "b"
ld c,1
jr z,set_drive

set_drive:
ld a,c
ld (drive),a
add a,'A'
call txt_output
;; delay some frames to see?
ret


drive_msg:
defb "Drive (A or B):",0

;;==================================================================
;; 
show_drive:
ld hl,drive_is_msg
call print_msg
ld a,(drive)
add a,&41
jp txt_output


drive_is_msg:
defb "Drive is ",0

;;==================================================================
;; debounce keyboard and return char pressed

debounce_char:
;; read keys while they are pressed
call km_read_char			
jr c,debounce_char

;; now wait for key to be pressed

debounce_char2:
call km_read_char
jr nc,debounce_char2
ret

;;==================================================================
;; debounce keyboard and return key code pressed

debounce_key:
call km_read_key
jr c,debounce_key
debounce_key2:
call km_read_key
jr nc,debounce_key2
ret

;;==================================================================

crlf:
ld a,10
call txt_output
ld a,13
call txt_output
ret

;;==================================================================
;; print message terminated with 0 char

print_msg:
ld a,(hl)
inc hl
or a
ret z
call txt_output
jr print_msg

;;==================================================================
;; print hex value

print_hex:
push af
;; show hex indicator
ld a,'&'
call txt_output
pop af

push af
rrca
rrca
rrca
rrca
call print_hex_digit
pop af
print_hex_digit:
and &f
cp 10
jr c,print_hex_digit2
add a,'A'-10
jp txt_output
print_hex_digit2:
add a,'0'
jp txt_output

;;==================================================================

cls:
;; set mode 2
ld a,2
jp scr_set_mode

;;==================================================================

;; text
on_txt:
defb "ON ",0

off_txt:
defb "OFF",0

;;===============================================
wait_vsyncs:
push bc
call mc_wait_flyback
halt
halt
pop bc
djnz wait_vsyncs
ret


;;===============================================

sense_drive_status:
ld a,%100						;; sense drive status
call fdc_write_command
ld a,(drive)					;; drive status of connected drive
call fdc_write_command
call fdc_result			;; status should show that drive is NOT READY!
ret

;;===============================================
;; send command to fdc
;;

;; 58 per command

fdc_write_command:
ld bc,&fb7e					;; [3]
push af						;; [4]
fwc1: in a,(c)				;; [4]
add a,a						;; [1]
jr nc,fwc1					;; [3]
add a,a						;; [1]
jr nc,fwc2					;; [3]
pop af						;; [1]
ret							

;; 12us before reading main status register

fwc2: pop af				;; [4]
inc c						;; [1]
out (c),a					;; [4]
dec c						;; [1]

;; delay before reading main status register
;; 12*2 = 24us

;; ex (sp),hl = 6
;; 6*4 = 24us
;;ex (sp),hl
;;ex (sp),hl
;;ex (sp),hl
;;ex (sp),hl					;; [24]

ld a,5				;; [2]
fwc3: dec a			;; [1]
jr nz,fwc3			;; [3] = (4*4)+2+2 = 20us
ret							;; [4]

;;===============================================
;; get result phase of command
;;
;; timing is not important here

fdc_result:
ld hl,result_data 
ld bc,&fb7e
fr1:
in a,(c)
cp #c0 
jr c,fr1 
inc c 
in a,(c) 
dec c 
ld (hl),a 
inc hl 

;;ex (sp),hl
;;ex (sp),hl
;;ex (sp),hl
;;ex (sp),hl
ld a,5 
fr2: 
dec a 
jr nz,fr2
in a,(c) 
and #10 
jr nz,fr1
ret 


;;----------------------------------------------------------
;; write data to fdc when in execution phase
;;
;; for commands that transfer data from CPU to FDC during execution phase
;; of command.

e_write1:
inc c					;; BC = I/O address for FDC data register
ld a,(de)				;; read byte from memory
out (c),a				;; output to FDC data register
dec c					;; BC = I/O address for FDC main status register
inc de					;; increment memory pointer

fdc_data_write:
in a,(c)				;; read fdc main status register
jp p,fdc_data_write			;; wait for fdc to signal it is ready to
						;; accept data 
						;; bit 7 will be 1 when FDC is ready to accept data


and &20					;; execution phase active? (bit 5 of main status register)
jr nz,e_write1
ret

;;-----------------------------------------------------------
;; read data from fdc when in execution phase
;;
;; for commands that transfer data from FDC to CPU during execution phase
;; of command.

e_read1:
inc c					;; BC = I/O address for FDC data register
in a,(c)				;; read from FDC data register
ld (de),a				;; write to memory
dec c					;; BC = I/O address for FDC main status register
inc de					;; increment memory pointer

fdc_data_read: 
in a,(c)				;; read fdc main status register
jp p,fdc_data_read			;; wait for fdc to signal it is ready to
						;; accept data
						;; bit 7 will be 1 when FDC is ready to accept data

and &20					;; execution phase active? (bit 5 of main status register)
jr nz,e_read1
ret

fdc_recalibrate:
ld b,2
fdc_recalibrate2:
push bc
;; seek to track 0
ld a,%111						;; recalibrate
call fdc_write_command
ld a,(drive)					;; drive
call fdc_write_command

fdc_recalibrate_check_end:
ld a,%1000						;; sense interrupt status
call fdc_write_command
call fdc_result

;; recalibrate completed?
ld ix,result_data
bit 5,(ix+0)
jr z,fdc_recalibrate_check_end
bit 4,(ix+0)
jr nz,fdc_recalibrate_ec
;; succeeded
pop bc
or a

ret

fdc_recalibrate_ec:
pop bc
djnz fdc_recalibrate2
;; failed after two attempts
scf
ret



fdc_seek:
;; issue seek command
ld a,%0000001111    ;; seek command
call fdc_write_command
ld a,(drive)
call fdc_write_command
ld a,(track)
call fdc_write_command

fdc_seek_check_end:
ld a,%1000						;; sense interrupt status
call fdc_write_command
call fdc_result

;; recalibrate completed?
ld ix,result_data
bit 5,(ix+0)
jr z,fdc_seek_check_end
bit 4,(ix+0)
jr nz,fdc_seek

ret

drive:
defb 0
sector:
defb 0
track:
defb 0

vsync_wait:
ld b,$f5
vs1:
in a,(c)
rra
jr nc,vs1
vs2:
in a,(c)
rra
jr c,vs2
ret


disc_motor_on:
;; this doesn't need interrupts disabled

;; turn on drive motor
ld bc,$fa7e
ld a,1
out (c),a

;; wait for 50 vsyncs to allow motor to achieve
;; max speed
ld c,50
dmo1:
call vsync_wait
dec c
jr nz,dmo1
ret

if 0
disc_motor_on:
;; turn on drive motor
ld bc,&fa7e
ld a,1
out (c),a

;; wait for motor to 
;; speed up and the speed to stablise
ld b,5
sdm_delay1:
ld hl,0
sdm_delay2:
dec hl
ld a,h
or l
jr nz,sdm_delay2
djnz sdm_delay1
ret
endif

result_data:
defs 7

end start